call sysproc.set_routine_opts('ISOLATION UR') 
@

CREATE OR REPLACE PROCEDURE DELETE_XACTIONS (
    IN I_TABNAME	CHARACTER(50),
    IN I_COMMIT_POINT	INTEGER,
    IN I_START_DATE TIMESTAMP  )

  SPECIFIC DELETE_XACTIONS
  DYNAMIC RESULT SETS 1
  LANGUAGE SQL
  NOT DETERMINISTIC
  EXTERNAL ACTION
  MODIFIES SQL DATA
  CALLED ON NULL INPUT
  INHERIT SPECIAL REGISTERS
-----------------------------------------------------------------------------
-- File:  DELETE_XACTIONS.SQL
--
--
-- Description:
--     Delete transactions from the SOR transaction tables.
--     Commit every i_commit_point rows to control the redo generation.
--
--
--
-- Called By:
--
--
-- Calls:
--     N/A
--
--
-- Notes:
--
--
-- Usage:
--     db2 "call DELETE_XACTIONS('FCM_CCCA', 10000, 120)"
--
--
-- History:
--     Date          User             Notes
--     2013/05/23    leushe           created 
--     2013/06/21    leushe           include VXX tables
--
------------------------------------------------------------------------
P1: BEGIN

    DECLARE l_del_count INTEGER DEFAULT 0;
    DECLARE l_rows_deleted INTEGER DEFAULT 0;
    DECLARE l_id BIGINT;
 

    DECLARE l_stmtTemplate varchar(1000);
    DECLARE l_stmtTemplateFact varchar(1000);    

    DECLARE l_delStmt varchar(1000);    

    DECLARE v_sqlstate CHAR(5);		              -- variable for SQLSTATE
    DECLARE v_sqlcode INTEGER;		              -- variable for SQLCODE
    DECLARE v_delete_count INTEGER     DEFAULT 0;     -- variable for deleted row count    
    
    DECLARE SQLCODE INTEGER DEFAULT 0;
    DECLARE SQLSTATE CHAR(5) DEFAULT '00000';
    
    DECLARE l_stmt STATEMENT;    

    DECLARE c_stmt CURSOR WITH HOLD FOR l_stmt;    
    
    DECLARE c_result CURSOR WITH RETURN FOR
       	SELECT i_tabname as TARGET_TAB
       	      ,v_delete_count as DELETE_ROWCOUNT       	      
       	      ,v_sqlstate as OUT_SQLSTATE
       	      ,v_sqlcode as OUT_SQLCODE
    	FROM SYSIBM.SYSDUMMY1;
    
    DECLARE CONTINUE HANDLER FOR SQLEXCEPTION
       SELECT SQLSTATE,
              SQLCODE
       INTO   v_sqlstate,
              v_sqlcode
       FROM SYSIBM.SYSDUMMY1;


    
    /* Initialize variables */
    VALUES (SQLSTATE, SQLCODE) INTO v_sqlstate, v_sqlcode;       

    SET l_stmtTemplate =
      'select cmx_tran_id from '|| rtrim(i_tabname) || ' where ( rqo_proc_utc_datetime <= ? )   FOR UPDATE' ;


    --
    -- cursor(s) to delete the rows returned from the driving cursor 
    --
    SET l_delStmt = 'DELETE FROM '|| rtrim(i_tabname) || ' WHERE CURRENT OF c_stmt ';  


    --
    -- sanity check that only the transaction tables are passed in
    --
    	If (NOT exists (select 1 
				from FCM_TRANSACTION_DEFINITION  
				where sor_Table_name = i_tabname
				      or sor_Table_name = substr(i_tabname,1,8)
					) )
		  THEN
                SET v_sqlcode = -1;
                SET v_sqlstate = '-1';
                SET v_delete_count = 0;             
    	OPEN c_result;
        RETURN -1;
    END IF;
      
    


    SET l_rows_deleted = 0; 
    SET l_del_count = 0;
    PREPARE l_stmt FROM l_stmtTemplate;
    OPEN c_stmt USING i_start_date ;

    SET v_sqlstate = SQLSTATE;
    WHILE ( v_sqlstate = '00000' ) DO

       FETCH c_stmt INTO l_id ;
       SET v_sqlstate = SQLSTATE;
       IF ( v_sqlstate = '00000' ) THEN
         EXECUTE IMMEDIATE l_delStmt;

          SET l_del_count = l_del_count + 1;
          IF ( l_del_count = i_commit_point ) THEN
             COMMIT;
             SET l_rows_deleted = l_rows_deleted + l_del_count;
             SET l_del_count = 0;
          END IF;

       END IF;

    END WHILE;

    IF ( l_del_count > 0 ) THEN
       COMMIT;
       SET l_rows_deleted = l_rows_deleted + l_del_count;
       SET v_delete_count = l_rows_deleted;
    END IF;

    CLOSE c_stmt;

    
    VALUES (SQLSTATE, SQLCODE) INTO v_sqlstate, v_sqlcode;
    
    OPEN c_result;

  END P1@

